% use 1-to-1 matched common peaks to fit model, the scatter plots look
% better, but the model is not significantly changed.
function step5_rescale_peak_strength_by_common_peaks(peak_name_1, peak_name_2, marker_name_1, marker_name_2, summit2summit_dist, workdir, MAtype, outputdir)
figcnt = 0;

% start of parameter region
marker_list = [ {marker_name_1}, {marker_name_2}];
marker_profix = MAtype;
extention = summit2summit_dist;

peak_list = [ {[peak_name_1 '_unique']}, ...  
            {[peak_name_2 '_unique']}, ...  
            {[peak_name_1 '_common']}, ...  
            {[peak_name_2 '_common']}, ...  
            ];
clr = 'bgkrym';
if_fit = [0,0,1,1];     % whether this set of peak will be used into robust fitting model
npeakset = length(peak_list);
% end of parameter region

legend_list = [];
% plot the MA plot before rescale 
figcnt = figcnt + 1;
figure1 = figure(figcnt);
clf;
axes1 = axes('Parent', figure1, 'YGrid', 'on');
box(axes1,'on');
hold(axes1,'all');
Amax = 0;
Amin = 10000;
for psidx = 1:npeakset
    TFname = peak_list{psidx};
    TFlegend = TFname;
    TFlegend(findstr(TFlegend, '_')) = ' ';
    legend_list = [legend_list, {TFlegend}];
        
    clear peak peak1;
    marker_name_1 = marker_list{1};
    peak_marker_file_1 = [workdir, TFname, '_peaks_', marker_name_1, '_read_density.mat'];
    load(peak_marker_file_1);     % peak
    peak1 = peak;
    
    clear peak peak2;
    marker_name_2 = marker_list{2};
    peak_marker_file_2 = [workdir, TFname, '_peaks_', marker_name_2, '_read_density.mat'];
    load(peak_marker_file_2);     % peak
    peak2 = peak;
    
    log2_peak1_read_density = log2(peak1.read_density+1);
    log2_peak2_read_density = log2(peak2.read_density+1);
    peak1.Mvalue = log2_peak1_read_density - log2_peak2_read_density;
    peak1.Avalue = (log2_peak1_read_density + log2_peak2_read_density)/2;
    scatter(peak1.Avalue, peak1.Mvalue, 30, clr(psidx));
    
    Amax = max(max(peak1.Avalue), Amax);
    Amin = min(min(peak1.Avalue), Amin);
    
    if if_fit(psidx) && sum(if_fit(1:psidx))==1      % first set of common peaks
        peak11 = peak1;
        peak12 = peak2;
        peaknum1 = length(peak1.chr_id);
    elseif if_fit(psidx) && sum(if_fit(1:psidx))==2      % first set of common peaks
        peak21 = peak1;
        peak22 = peak2;
        peaknum2 = length(peak1.chr_id);
    end
end
xlabel('A value');
ylabel('M value');
legend(legend_list);
title('before rescale');

% select matched common peaks to fit model
peak11.response = zeros(peaknum1, 1);
peak21.response = zeros(peaknum2, 1);
chr_id_str = unique(peak11.chr_id);
for i = 1:length(chr_id_str)
    if peaknum1<=peaknum2
        I1 = find(strcmp(peak11.chr_id, chr_id_str{i}));
        bpsummit_t1 = peak11.bpsummit(I1);
        bpstart_t1 = bpsummit_t1 - extention;
        bpend_t1 = bpsummit_t1 + extention;

        I2 = find(strcmp(peak21.chr_id, chr_id_str{i}));
        bpsummit_t2 = peak21.bpsummit(I2);
        
        for m = 1:length(bpstart_t1)
            I = find((bpsummit_t2-bpend_t1(m)).*(bpsummit_t2-bpstart_t1(m))<=0);
            if ~isempty(I)  %overlap
                 peak11.response(I1(m)) = 1;
                 summit_dist = abs(bpsummit_t2(I)-bpsummit_t1(m));
                 [mindist, Im] = min(summit_dist);
                 peak21.response(I2(I(Im))) = 1;
            end
        end
    else
        I1 = find(strcmp(peak11.chr_id, chr_id_str{i}));
        bpsummit_t1 = peak11.bpsummit(I1);

        I2 = find(strcmp(peak21.chr_id, chr_id_str{i}));
        bpsummit_t2 = peak21.bpsummit(I2);
        bpstart_t2 = bpsummit_t2 - extention;
        bpend_t2 = bpsummit_t2 + extention;
        
        for m = 1:length(bpstart_t2)
            I = find((bpsummit_t1-bpend_t2(m)).*(bpsummit_t1-bpstart_t2(m))<=0);
            if ~isempty(I)  %overlap
                 peak21.response(I2(m)) = 1;
                 summit_dist = abs(bpsummit_t1(I)-bpsummit_t2(m));
                 [mindist, Im] = min(summit_dist);
                 peak11.response(I1(I(Im))) = 1;
            end
        end
    end
end

I1 = find(peak11.response);
I2 = find(peak21.response);
fit_X = [peak11.Avalue(I1); peak21.Avalue(I2)];
fit_Y = [peak11.Mvalue(I1); peak21.Mvalue(I2)];
read_density_1 = [{peak11.read_density(I1)+1}, {peak21.read_density(I2)+1}];
read_density_2 = [{peak12.read_density(I1)+1}, {peak22.read_density(I2)+1}];

% calculate the total number of read in matched common peaks
read_count_1 = sum(peak11.read_density(I1)+1);
read_count_2 = sum(peak22.read_density(I2)+1);
        
% fit the model
Isel = find(fit_Y>=-10 & fit_Y<=10);
b = robustfit(fit_X(Isel), fit_Y(Isel));
if b(2) >= 0
    disp(['Model for normalization: M = ' num2str(b(1)) ' + ' num2str(b(2)) ' * A']);
else
    disp(['Model for normalization: M = ' num2str(b(1)) ' - ' num2str(-b(2)) ' * A']);
end

% plot the fitting model into figure 1
px = floor(Amin):0.1:floor(Amax)+1;
py = b(1) + b(2) * px;
plot(px, py, 'r', 'parent', axes1, 'linewidth', 2);
saveas(gcf, [outputdir, marker_profix, '_MA_plot_pre_MA_rescale.fig'], 'fig');
saveas(gcf, [outputdir, marker_profix, '_MA_plot_pre_MA_rescale.tif'], 'tif');

% plot the scatter plots of read count in common peaks between two chip-seq sets
if 1
    figcnt = figcnt + 1;
    figure(figcnt)
    clf;
    hold on;
    Ifit = find(if_fit);
    RDmin = 10000;
    RDmax = 0;
    for i = 1:sum(if_fit)
        RDmax = max(max(log2(read_density_1{i})), RDmax);
        RDmin = min(min(log2(read_density_1{i})), RDmin);
        scatter(log2(read_density_1{i}), log2(read_density_2{i}), 30, clr(Ifit(i)));
    end
    xlabstr = marker_list{1};
    xlabstr(findstr(xlabstr, '_')) = ' ';
    xlabel(xlabstr);
    ylabstr = marker_list{2};
    ylabstr(findstr(ylabstr, '_')) = ' ';
    ylabel(ylabstr);
    legend(legend_list(Ifit))
    title('Log2 Read Density');
    rx = floor(RDmin):0.1:floor(RDmax)+1;
    ry = (2-b(2))*rx/(2+b(2)) - 2*b(1)/(2+b(2));
    plot(rx, ry, 'r', 'linewidth', 2);
    saveas(gcf, [outputdir, marker_profix, '_ReadCount_scatter_plot_with_MA_fit.fig'], 'fig');
    saveas(gcf, [outputdir, marker_profix, '_ReadCount_scatter_plot_with_MA_fit.tif'], 'tif');
end

% plot the MA plot after rescale 
figcnt = figcnt + 1;
figure4 = figure(figcnt);
clf;
axes4 = axes('Parent', figure4, 'YGrid', 'on');
box(axes4, 'on');
hold(axes4, 'all');
for psidx = 1:npeakset
    TFname = peak_list{psidx};
        
    clear peak peak1;
    marker_name_1 = marker_list{1};
    peak_marker_file_1 = [workdir, TFname, '_peaks_', marker_name_1, '_read_density.mat'];
    load(peak_marker_file_1);     % peak
    peak1 = peak;
    
    clear peak peak2;
    marker_name_2 = marker_list{2};
    peak_marker_file_2 = [workdir, TFname, '_peaks_', marker_name_2, '_read_density.mat'];
    load(peak_marker_file_2);     % peak
    peak2 = peak;
    
    clear peak;
    peak_file = [workdir, TFname, '_bound_region.mat'];
    load(peak_file);        % peak
    npeak = length(peak.chr_id);
    
    % calculate normalized MA value
    log2_peak1_read_density = log2(peak1.read_density+1);
    log2_peak2_read_density = log2(peak2.read_density+1);
    Mvalue = log2_peak1_read_density - log2_peak2_read_density;
    Avalue = (log2_peak1_read_density + log2_peak2_read_density)/2;
    log2_peak1_read_density_rescale = (2-b(2)).*log2_peak1_read_density/(2+b(2)) - 2*b(1)/(2+b(2));
    peak.Mvalue_rescaled = log2_peak1_read_density_rescale - log2_peak2_read_density;
    peak.Avalue_rescaled = (log2_peak1_read_density_rescale + log2_peak2_read_density)/2;
    
    % calculate p-value
    peak.pvalue_MAvalue = NaN(npeak, 1);
    for kk = 1:npeak
        read_density_norm_1 = 2^log2_peak1_read_density_rescale(kk);
        read_density_norm_2 = 2^log2_peak2_read_density(kk);
        peak.pvalue_MAvalue(kk) = step0_digit_exprs_p_norm(read_density_norm_1, read_density_norm_2);
        %peak.pvalue_MAvalue(kk) = step0_digit_exprs_p(peak1.read_density(kk)+1, peak2.read_density(kk)+1, read_count_1, read_count_2);
    end
    
    % generate MA plot for this set of peaks together with p-value
    scatter(peak.Avalue_rescaled, peak.Mvalue_rescaled, 30, clr(psidx));
    
    if if_fit(psidx) && sum(if_fit(1:psidx))==1      % first set of common peaks
        peak11 = peak;
    elseif if_fit(psidx) && sum(if_fit(1:psidx))==2      % first set of common peaks
        peak21 = peak;
    end
    
    outfile = [workdir, TFname, '_peaks_', marker_profix, '_MAvalue.mat'];
    save(outfile, 'peak');
    
    outtable = [outputdir, TFname, '_peaks_', marker_profix, '_MAvalue.xls'];
    fw = fopen(outtable, 'w');
    fprintf(fw, 'Chromosome\tStart\tEnd\tSummit\tPeak1_rawread_density\tPeak2_rawread_density\tM_value\tA_value\t-log10_Pvalue_of_differential_binding\n');
    for kk = 1:npeak
        fprintf(fw, '%s\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n', peak.chr_id{kk}, peak.bpstart(kk), peak.bpend(kk), peak.bpsummit(kk)-peak.bpstart(kk)+1, peak1.read_density(kk), peak2.read_density(kk),peak.Mvalue_rescaled(kk), peak.Avalue_rescaled(kk), -log10(peak.pvalue_MAvalue(kk)));
    end
    fclose(fw);
end
xlabel('A value');
ylabel('M value');
legend(legend_list);
title('after rescale');
saveas(gcf, [outputdir, marker_profix, '_MA_plot_post_MA_rescale.fig'], 'fig');
saveas(gcf, [outputdir, marker_profix, '_MA_plot_post_MA_rescale.tif'], 'tif');

% plot the P-value after rescale 
figcnt = figcnt + 1;
figure5 = figure(figcnt);
clf;
axes5 = axes('Parent', figure5, 'YGrid', 'on');
box(axes5, 'on');
hold(axes5, 'all');
for psidx = 1:npeakset
    clear peak;
    TFname = peak_list{psidx};
    peak_file = [workdir, TFname, '_peaks_', marker_profix, '_MAvalue.mat'];
    load(peak_file);        % peak
    
    % generate MA plot for this set of peaks together with p-value
    scatter(peak.Avalue_rescaled, peak.Mvalue_rescaled, 'CData', min(-log10(peak.pvalue_MAvalue), 50));
end
colorbar;
xlabel('A value');
ylabel('M value');
title('-log_{10}(P-value)');
saveas(gcf, [outputdir, marker_profix, '_MA_plot_with_Pvalue.fig'], 'fig');
saveas(gcf, [outputdir, marker_profix, '_MA_plot_with_Pvalue.tif'], 'tif');

